VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsTimer"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' ***************************************************************************
' Module:        clsTimer (Formerly clsSlumber)
'
' Description:   This class encapsulates the QueryPerfomanceXXX (or the
'              GetTickcount) and the Sleep API functions to put the thread
'              of your application to Sleep for a period of time.  The
'              Sleep API is preferable to a loop since it does not consume
'              processor time.  While sleeping, no events can be received
'              by the application.  The Milliseconds() routine provides an
'              encapsulated alternative to Sleep and will Sleep for small
'              intervals at a time.  After each return from Sleep, it will
'              call DoEvents so that your application can receive events.
'              It also raises its own Milliseconds event so that if you can
'              have your application check on something each Slumber.  If
'              you want to Exit out of the Slumber period during one of
'              these events, you can use the StopProcessing property to
'              notify the class that it is time to stop processing.
'              Otherwise, clsTimer will put the thread asleep again until
'              either a StopProcessing command is indicated or the
'              calculation period has finished.  The cSlumberInterval property
'              allows you to set the interval time for the Sleep cycles, by
'              default this is set for 0.5 seconds or 500 milliseconds.  If
'              you are using a high performance timer, then this class
'              measures time to a precision of 0.0001 seconds which is 10
'              times better than the timeGetTime() or GetTickCount() APIs
'              which have a precision of 0.1 seconds.  If you do not have a
'              high performance timer, then this class will use the
'              GetTickcount() API.
'
' Application: This class is very versatile. It can be used as a simple way
'              to put your app on idle while it is waiting for something,
'              or it can be used to keep track of complicated asynchronous
'              processes.
'
' ===========================================================================
'    DATE      NAME / DESCRIPTION
' -----------  --------------------------------------------------------------
' 01-FEB-2000  David Drake
'              Original module using timeGetTime() API
' 03-MAR-2000  David Drake
'              Added Slumber() event
' 13-DEC-2000  David Drake
'              Changed class to use QueryPerformanceCounter() API and
'              added the ElapsedMilliseconds Property
' 20-APR-2001  Kenneth Ives  kenaso@home.com
'              Added properties for Hours, Minutes, Seconds, TotalTime.
'              Changed WakeUp method to a property (cStopProcessing) to
'              allow the user to activate/deactivate the timing process.
'              Added documentation to all the routines.  Added the
'              functionality to determine if there is access to a high
'              performance timer.  If not, then use the GetTickCount() API
'              to process the milliseconds.  Removed some of the redundant
'              error routines.  Renamed Slumber() to Milliseconds().
' 24-APR-2001  Kenneth Ives  kenaso@home.com
'              Added property for Days. Modified the formatted output string.
'              Renamed property ...StartSeconds to ...Milliseconds for
'              clarity.  Moved examples to a dummy routine named Examples().
' ***************************************************************************
Option Explicit

' --------------------------------------------------------------------------
' Constants
' --------------------------------------------------------------------------
  Private Const CONST_2_31  As Double = 2147483648#  ' 2^31
  Private Const CONST_2_32  As Double = 4294967296#  ' 2^32

' --------------------------------------------------------------------------
' Type structures
' --------------------------------------------------------------------------
  Private Type Large_Integer
      Low   As Long
      High  As Long
  End Type

' --------------------------------------------------------------------------
' Declarations
' --------------------------------------------------------------------------
  ' Sleep pauses program execution for a certain amount of time
  ' (in milliseconds). This is more accurate than using a do-nothing loop,
  ' waiting for a certain amount of time to pass.
  Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

  ' QueryPerformanceCounter obtains the current reading of the system's
  ' high-performance timer. The high-performance timer is a counter that
  ' the system increments many times a second. To find out how quickly
  ' the timer is incremented, use QueryPerformanceFrequency. If you know
  ' its frequency, you can use the high-performance timer to measure small
  ' intervals of time precisely. However, keep in mind that not all
  ' computers support a high-performance timer.
  Private Declare Function QueryPerformanceCounter Lib "kernel32" _
          (lpPerformanceCount As Large_Integer) As Long

  ' QueryPerformanceFrequency obtains the frequency of the system's
  ' high-performance timer. This value indicates the number of times the
  ' high-performance timer increments itself every second. Knowing this
  ' frequency, you can use QueryPerformanceCounter to measure precise time
  ' intervals. However, not all systems support a high-performance timer.
  Private Declare Function QueryPerformanceFrequency Lib "kernel32" _
          (lpFrequency As Large_Integer) As Long

  ' This is a rough translation of the GetTickCount API. The tick count of
  ' a PC is only valid for the first 49.7 days since the last reboot.
  ' When you capture the tick count, you are capturing the total number of
  ' milliseconds elapsed since the last reboot.
  Private Declare Function GetTickCount Lib "kernel32" () As Long
  
' --------------------------------------------------------------------------
' Module level variables
' --------------------------------------------------------------------------
  Private m_blnStopProcessing  As Boolean
  Private m_blnHighPerf        As Boolean
  Private m_blnRestart         As Boolean
  Private m_intPercent         As Integer
  Private m_lngInterval        As Long
  Private m_lngStart           As Long
  Private m_dblFrequency       As Double
  Private m_dblFinishCount     As Double
  Private m_dblTotalTime       As Double
  Private m_dblMilliseconds    As Double
  Private m_dblCurrentTime     As Double
  Private m_dblDays            As Double
  Private m_dblHours           As Double
  Private m_dblMinutes         As Double
  Private m_dblSeconds         As Double
  Private m_dblElapsedSeconds  As Double
  Private m_strOutputFormat    As String
  Private m_typStartingCount   As Large_Integer

' -----------------------------------------------------------------------------
' Public Event to return the current formatted time display and the percentage
' of completion.
' -----------------------------------------------------------------------------
  Public Event cMilliseconds(strOutput As String, intPercent As Integer)

Private Sub Examples()

' Example 1:   This is an example for idling your application.
'              This line is in the Declaration section of the calling module
'              or form.
'
'              Private WithEvents mobjSlumber As clsTimer
'
'              Private Sub RunProcess()
'                Set mobjSlumber = New clsTimer
'                mobjSlumber.StopProcessing = False
'
'                Do
'                   If mbWorkToDo Then
'                       Call ProcessWork()
'                   Else
'                       mobjSlumber.Slumber = 5000  ' Sleep for 5 seconds
'                   End If
'
'                   If mbStop Then Exit Do
'                Loop
'
'                Set mobjSlumber = nothing
'              End Sub
'
'              This routine will be created but you have to add the code.
'
'              Private Sub mobjSlumber_Slumber(strOutput as string, _
'                                              intPercent as integer)
'                ' Now every 0.5 seconds, it will check
'                ' to StopProcessing
'                If mbStop or mbWorkToDo Then
'                    mobjSlumber.StopProcessing = True
'                End If
'              End Sub
'
' Example 2:   This is an example for asyncronous processing.
'              Private WithEvents mobjSlumber As clsTimer
'
'              Private Const GETDATA_TIMEOUT = 60000 'Timeout after 1 minute
'
'              Private Sub GetData()
'                Set mobjSlumber = New clsTimer
'                mobjSlumber.StopProcessing = False
'                Call mobjAsynchGetData.Start()
'
'                mobjSlumber.Slumber = GETDATA_TIMEOUT ' Slumber for 1 minute
'
'                If Not mobjAsynchGetData.DataArrived Then

'                    Err.Raise vbObjectError, _
'                              "GetData", _
'                              "A Timeout occurred while trying to get data!"
'                End if
'
'                  Set mobjSlumber = nothing
'              End Sub
'
'              Private Sub mobjSlumber_Slumber(strOutput as string, _
'                                              intPercent as integer)
'                ' Every minute, it will check for data arrival
'                If mobjAsynchGetData.DataArrived Then
'                    mobjSlumber.StopProcessing = True
'                Elseif
'                    ' You can call back into clsTimer to figure
'                    ' out the percentage of time elapsed
'                    Debug.Print Format$(intPercent, "Percent") & " to Timeout"
'                End If
'              End Sub
 
End Sub

' *****************************************************************************
' **                     Property Routines
' *****************************************************************************
Public Property Let cTotalTime(ByVal dblData As Double)

' -----------------------------------------------------------------------------
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cTotalTime = 5
' -----------------------------------------------------------------------------
  m_dblTotalTime = dblData

End Property

Public Property Let cMilliseconds(ByVal dblData As Double)

' -----------------------------------------------------------------------------
' Starting time in milliseconds.
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cMilliseconds = 5
' -----------------------------------------------------------------------------
  m_dblMilliseconds = dblData

End Property

Public Property Get cMilliseconds() As Double

' -----------------------------------------------------------------------------
' Starting time in milliseconds.
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: Debug.Print X.cMilliseconds
' -----------------------------------------------------------------------------
  cMilliseconds = m_dblMilliseconds

End Property

Public Property Let cSeconds(ByVal dblData As Double)

' -----------------------------------------------------------------------------
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cSeconds = 5
' -----------------------------------------------------------------------------
  m_dblSeconds = dblData

End Property

Public Property Get cSeconds() As Double

' -----------------------------------------------------------------------------
' used when retrieving value of a property, on the right side of an assignment.
' Syntax: Debug.Print X.cSeconds
' -----------------------------------------------------------------------------
  cSeconds = m_dblSeconds

End Property

Public Property Let cMinutes(ByVal dblData As Double)

' -----------------------------------------------------------------------------
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cMinutes = 5
' -----------------------------------------------------------------------------
  m_dblMinutes = dblData

End Property

Public Property Get cMinutes() As Double

' -----------------------------------------------------------------------------
' used when retrieving value of a property, on the right side of an assignment.
' Syntax: Debug.Print X.cMinutes
' -----------------------------------------------------------------------------
  cMinutes = m_dblMinutes

End Property

Public Property Let cHours(ByVal dblData As Double)

' -----------------------------------------------------------------------------
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cHours = 5
' -----------------------------------------------------------------------------
  m_dblHours = dblData

End Property

Public Property Get cHours() As Double

' -----------------------------------------------------------------------------
' used when retrieving value of a property, on the right side of an assignment.
' Syntax: Debug.Print X.cHours
' -----------------------------------------------------------------------------
  cHours = m_dblHours

End Property

Public Property Let cDays(ByVal dblData As Double)

' -----------------------------------------------------------------------------
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cDays = 5
' -----------------------------------------------------------------------------
  m_dblDays = dblData

End Property

Public Property Get cDays() As Double

' -----------------------------------------------------------------------------
' used when retrieving value of a property, on the right side of an assignment.
' Syntax: Debug.Print X.cDays
' -----------------------------------------------------------------------------
  cDays = m_dblDays

End Property

Public Property Get cSlumberInterval() As Long
  
' -----------------------------------------------------------------------------
' Save the timer interval to an external available property
' -----------------------------------------------------------------------------
  cSlumberInterval = m_lngInterval
              
End Property

Public Property Let cSlumberInterval(lngData As Long)
  
' -----------------------------------------------------------------------------
' If the slumber interval count drops to zero then reset to 10
' -----------------------------------------------------------------------------
  m_lngInterval = lngData
    
End Property

Public Property Get cStopProcessing() As Boolean
  
' -----------------------------------------------------------------------------
' Used to to Start/Stop processing.  (True/False)
' used when retrieving value of a property, on the right side of an assignment.
' Syntax: Debug.Print X.cStopProcessing
' -----------------------------------------------------------------------------
  cStopProcessing = m_blnStopProcessing
              
End Property

Public Property Let cStopProcessing(blnData As Boolean)
  
' -----------------------------------------------------------------------------
' Used to to Start/Stop processing.  (True/False)
' used when assigning a value to the property, on the left side of an
' assignment.     Syntax: X.cStopProcessing = True
' -----------------------------------------------------------------------------
  m_blnStopProcessing = blnData
    
End Property

Public Property Get ElapsedMilliseconds() As Double
  
' -----------------------------------------------------------------------------
' Calculate the number of elapsed milliseconds.
' -----------------------------------------------------------------------------
  ElapsedMilliseconds = CDbl((m_dblElapsedSeconds * 10000) / 10)
            
End Property

' *****************************************************************************
' **                     Calculation Routines
' *****************************************************************************
Private Function Calc_Difference(typCurrentCount As Large_Integer, _
                                 typStartingCount As Large_Integer) As Double
  
' -----------------------------------------------------------------------------
' Determine the number of elapsed milliseconds by calculating the difference
' between the Start time and the projected Finish time.
' -----------------------------------------------------------------------------
  With typStartingCount
       If .Low > 0& Then
           If typCurrentCount.Low > 0& Then
               Calc_Difference = typCurrentCount.Low - .Low + (typCurrentCount.High - .High) * CONST_2_32
           Else
               Calc_Difference = CONST_2_31 + CDbl(typCurrentCount.Low And &H7FFFFFFF) - .Low + (typCurrentCount.High - .High) * CONST_2_32
           End If
       Else
           If typCurrentCount.Low > 0& Then
               Calc_Difference = typCurrentCount.Low - CONST_2_31 - CDbl(.Low And &H7FFFFFFF) + (typCurrentCount.High - .High) * CONST_2_32
           Else
               Calc_Difference = CDbl(typCurrentCount.Low And &H7FFFFFFF) - CDbl(.Low And &H7FFFFFFF) + (typCurrentCount.High - .High) * CONST_2_32
           End If
       End If
  End With
    
End Function

Private Sub Calc_Format()

' -----------------------------------------------------------------------------
' Define local variables
' -----------------------------------------------------------------------------
  Dim dblElapsedMilliseconds As Double
  
' -----------------------------------------------------------------------------
' Capture number of elpased milliseconds once so we do not have to make an
' extra call to another routine.
' -----------------------------------------------------------------------------
  dblElapsedMilliseconds = ElapsedMilliseconds
  
' -----------------------------------------------------------------------------
' clsTimer module passes the number of milliseconds that have elapsed to
' this routine via the WithEvents.  The first time thru, we use the original
' calculated time.
' -----------------------------------------------------------------------------
  If dblElapsedMilliseconds > 0 Then
      ' Calculate the percent of the elapsed time
      m_intPercent = Int((dblElapsedMilliseconds / m_dblMilliseconds) * 100)
  
      ' calculate time elapsed
      m_dblCurrentTime = m_dblTotalTime - (dblElapsedMilliseconds / 1000)
  Else
      ' first time thru, use the value passed to this class
      m_dblCurrentTime = m_dblTotalTime
  End If
    
' -----------------------------------------------------------------------------
' Calculate the countdown
' -----------------------------------------------------------------------------
  m_dblDays = Int(m_dblCurrentTime / 86400)
  m_dblSeconds = m_dblCurrentTime - (m_dblDays * 86400)
  m_dblHours = Int(m_dblSeconds / 3600)
  m_dblSeconds = m_dblSeconds - (m_dblHours * 3600)
  m_dblMinutes = Int(m_dblSeconds / 60)
  m_dblSeconds = m_dblSeconds - (m_dblMinutes * 60)
        
' -----------------------------------------------------------------------------
' Update countdown display based on if using a high performance timer or not
' -----------------------------------------------------------------------------
  m_strOutputFormat = Format$(m_dblDays, "00") & ":" & _
                      Format$(m_dblHours, "00") & ":" & _
                      Format$(m_dblMinutes, "00") & ":"
                      
  If m_blnHighPerf Then
      ' Formatted to tenThousandths of a second
      m_strOutputFormat = m_strOutputFormat & Format$(m_dblSeconds, "00.0000")
  Else
      ' Formatted to thousandths of a second
      m_strOutputFormat = m_strOutputFormat & Format$(m_dblSeconds, "00.000")
  End If

End Sub

Private Function ConvertTypeToDouble(typLargeNumber As Large_Integer) As Double
  
' -----------------------------------------------------------------------------
' Convert the type structure for a Large_Integer type into a double precision
' number
' -----------------------------------------------------------------------------
  With typLargeNumber
       If .Low > 0& Then
           ConvertTypeToDouble = .Low + (.High * CONST_2_32)
       Else
           ConvertTypeToDouble = CONST_2_31 + CDbl(.Low And &H7FFFFFFF) + (.High * CONST_2_32)
       End If
  End With

End Function

' *****************************************************************************
' **                     Event Routine
' *****************************************************************************
Public Sub Milliseconds()
  
  On Error GoTo Milliseconds_Error
' -----------------------------------------------------------------------------
' Define local variables
' -----------------------------------------------------------------------------
  Dim lngCurrent       As Long
  Dim typCurrentCount  As Large_Integer
    
' -----------------------------------------------------------------------------
' If the millisecond count drops to zero then raise an error message
' -----------------------------------------------------------------------------
  If m_dblMilliseconds <= 0 Then
      Err.Raise vbObjectError, _
                "[clsTimer.Milliseconds]", _
                "Parameter MilliSeconds must be greater than 0!"
      Exit Sub    ' safety net in case we return here by mistake
  End If
    
' -----------------------------------------------------------------------------
' Convert the number of milliseconds into whole seconds.  Initialize the
' elapsed millisecond counter.
' -----------------------------------------------------------------------------
  m_dblFinishCount = m_dblMilliseconds / 1000
  m_dblElapsedSeconds = 0
  
' -----------------------------------------------------------------------------
' Get the starting time in milliseconds based on whether or not there is a
' high performance timer available.
' -----------------------------------------------------------------------------
  If m_blnHighPerf Then
            
      ' Get the current number of milliseconds since
      ' the last reboot as our starting point
      QueryPerformanceCounter m_typStartingCount
  
  Else ' No high performance timer available
            
      ' Get the current number of milliseconds since
      ' the last reboot as our starting point
      m_lngStart = GetTickCount()
  End If
  
' -----------------------------------------------------------------------------
' Loop and processing the millisecond count until we reach the projected end or
' the StopProcessing flag is set to TRUE.
' -----------------------------------------------------------------------------
  Do
      ' pause for a specified time
      Sleep m_lngInterval
          
      ' Calculate the elapsed time and the percentage
      ' of completion.  Then place into an acceptable
      ' format.
      Call Calc_Format
      
      On Error GoTo Milliseconds_Event_Error
      ' Raise current elapsed milliseconds
      ' to the module with the WithEvents
      RaiseEvent cMilliseconds(m_strOutputFormat, m_intPercent)
      DoEvents
          
      On Error GoTo Milliseconds_Error
      ' See if we have a high performance timer
      If m_blnHighPerf Then
            
          ' Get the current number of milliseconds
          QueryPerformanceCounter typCurrentCount
        
          ' calclate the difference between the starting
          ' count and the current count
          m_dblElapsedSeconds = Calc_Difference(typCurrentCount, m_typStartingCount) / m_dblFrequency
          
      Else ' No high performance timer available
                      
          ' Get the current number of milliseconds
          lngCurrent = GetTickCount()
            
          ' calclate the difference between the starting
          ' count and the current count
          m_dblElapsedSeconds = (lngCurrent - m_lngStart) / 1000
              
      End If
      
      ' if the projected number of milliseconds have elapsed or
      ' the StopProcessing flag is TRUE then exit this loop because we
      ' are finished.
      If m_dblElapsedSeconds >= m_dblFinishCount Or m_blnStopProcessing Then
          Exit Do
      End If
  Loop
  
' -----------------------------------------------------------------------------
' If the stop processing was selected then exit this routine
' -----------------------------------------------------------------------------
  If m_blnStopProcessing Then
      Exit Sub
  End If
  
' -----------------------------------------------------------------------------
' Save the number of projected milliseconds
' -----------------------------------------------------------------------------
  m_dblElapsedSeconds = m_dblFinishCount
      
' -----------------------------------------------------------------------------
' Calculate the elapsed time and the percentage of completion.  Then place into
' an acceptable format.
' -----------------------------------------------------------------------------
  Call Calc_Format
      
' -----------------------------------------------------------------------------
' Raise current elapsed milliseconds to the module with the WithEvents
' -----------------------------------------------------------------------------
  RaiseEvent cMilliseconds(m_strOutputFormat, m_intPercent)
  DoEvents
  Exit Sub
    
' -----------------------------------------------------------------------------
' If a TIMING error occurs then raise an error message
' -----------------------------------------------------------------------------
Milliseconds_Error:
  Err.Raise Err.Number, Err.Source, _
            "[clsTimer.Milliseconds]" & Err.Description
  Exit Sub    ' safety net in case we return here by mistake
    
' -----------------------------------------------------------------------------
' If an EVENT error occurs then raise an error message
' -----------------------------------------------------------------------------
Milliseconds_Event_Error:
  Err.Raise Err.Number, Err.Source, _
            "[clsTimer.Milliseconds][Milliseconds.Event]" & Err.Description
            
End Sub

' *****************************************************************************
' **           Class Module Initialization and Termination Routines
' *****************************************************************************
Private Sub Class_Initialize()
    
' -----------------------------------------------------------------------------
' Define local variables
' -----------------------------------------------------------------------------
  Dim typPerfFrequency As Large_Integer
  
' -----------------------------------------------------------------------------
' See If there is a high performance timer available.
' -----------------------------------------------------------------------------
  QueryPerformanceFrequency typPerfFrequency
  
' -----------------------------------------------------------------------------
' Convert data in Large_Integer type structure to a double precision number.
' -----------------------------------------------------------------------------
  m_dblFrequency = ConvertTypeToDouble(typPerfFrequency)
  
' -----------------------------------------------------------------------------
' If we have a value greater than 0 then we are using a high performance
' timer; otherwise, we will use the GetTickCount() API.  We temporarily
' convert the double precision number to a long for testing because
' somewhere in the 'nth' decimal position there may be a digit that is not
' a zero.
' -----------------------------------------------------------------------------
  If CLng(m_dblFrequency) > 0 Then
      ' A high performance timer is available.
      m_blnHighPerf = True
  Else
      ' A high performance timer is NOT available.
      m_blnHighPerf = False
  End If
    
' -----------------------------------------------------------------------------
' Set the Slumber interval to half a second  (1000 milliseconds = 1 second).
' This way there is a pause in the processing to react to outside events.
' -----------------------------------------------------------------------------
  m_lngInterval = 500

End Sub

Private Sub Class_Terminate()

' -----------------------------------------------------------------------------
' Define local varialbes
' -----------------------------------------------------------------------------
  Dim typDummy As Large_Integer
    
' -----------------------------------------------------------------------------
' Empty module level variables
' -----------------------------------------------------------------------------
  m_lngInterval = 0
  m_intPercent = 0
  m_lngStart = 0
  m_dblFrequency = 0
  m_dblFinishCount = 0
  m_dblTotalTime = 0
  m_dblMilliseconds = 0
  m_dblCurrentTime = 0
  m_dblDays = 0
  m_dblHours = 0
  m_dblMinutes = 0
  m_dblSeconds = 0
  m_dblElapsedSeconds = 0
  m_strOutputFormat = ""
  m_typStartingCount = typDummy   ' Empties type structure with null values
    
End Sub
